from typing import Iterable, Dict, Tuple
import json
import requests

from .adaptive_types import JsonObject
from .adaptive_auth import AdaptiveAuth

FileTuple = Tuple[str, str | bytes, str]  # (filename, filecontent, mimetype)


class AdaptiveImport:
    """Class to handle data import operations with Adaptive."""

    _tenant: str | None
    _adaptive_auth: AdaptiveAuth | None
    _url: str

    def __init__(
        self,
        url: str,
        adaptive_auth: AdaptiveAuth | None = None,
        tenant: str | None = None,
    ):
        """Initialize the AdaptiveImport class.

        Args:
            url (str): The base URL for the Adaptive OGC API.
            adaptive_auth (AdaptiveAuth): An instance of AdaptiveAuth for authentication.
            tenant (str | None, optional): The tenant identifier for multi-tenant setups. Defaults to None.
        """
        self._url = url.rstrip("/") + "/api/import/v1/dataset/"
        self._adaptive_auth = adaptive_auth
        self._tenant = tenant

    def _get_headers(self) -> Dict[str, str]:
        headers: Dict[str, str] = {}
        if self._adaptive_auth is not None:
            token = self._adaptive_auth.get_token()
            headers["Authorization"] = f"Bearer {token}"

        if self._tenant is not None:
            headers["X-Gaia-Tenant"] = self._tenant

        return headers

    def import_items(
        self,
        dataset: str,
        items: Iterable[JsonObject],
        options: JsonObject | None = None,
    ) -> requests.Response:
        """Import items to a specified dataset. The items are plain JSON features. Geometries need to be WKT.

        Args:
            dataset (str): The dataset alias to import the items into.
            items (Iterable[JsonDict]): An iterable of items (features) to be imported.
            options (JsonDict): Additional import options as a dictionary.

        Returns:
            requests.Response: The response object from the import request.
        """
        file = ("data.json", json.dumps(items), "application/json")
        return self.import_file(dataset, file, options)

    def import_file(
        self,
        dataset: str,
        files: FileTuple | list[FileTuple],
        options: JsonObject | None = None,
    ) -> requests.Response:
        """Import a geodata file (or files, for unzipped Shapefile) to a specified dataset.

        Args:
            dataset (str): The dataset alias to import the file(s) into.
            files (FileTuple | list[FileTuple]): A single file tuple or a list of file tuples to be imported.
            options (JsonDict): Additional import options as a dictionary.

        Returns:
            requests.Response: The response object from the import request.
        """
        url = self._url + dataset
        headers = self._get_headers()

        files_req = {}
        if isinstance(files, tuple):
            files_req["files"] = files
        elif len(files) == 1:
            files_req["files"] = files[0]
        else:
            for i, file in enumerate(files):
                files_req[f"files[{i}]"] = file

        data = {"options": json.dumps(options or {})}

        req = requests.Request(
            "POST",
            url,
            headers=headers,
            files=files_req,
            data=data,
        )
        prepared = req.prepare()

        response = requests.Session().send(prepared)
        return response



import time
import os
from dotenv import load_dotenv
from pyadaptive.adaptive_auth import AdaptiveAuth

load_dotenv()


def get_auth_instance() -> AdaptiveAuth:
    url = os.getenv("KEYCLOAK_URL")
    client_id = os.getenv("KEYCLOAK_CLIENT_ID")
    client_secret = os.getenv("KEYCLOAK_CLIENT_SECRET")
    realm_name = os.getenv("KEYCLOAK_REALM_NAME")

    assert url is not None
    assert client_id is not None
    assert client_secret is not None
    assert realm_name is not None

    auth = AdaptiveAuth(
        server_url=url,
        client_id=client_id,
        client_secret_key=client_secret,
        realm_name=realm_name,
    )
    return auth


def test_update_token():
    auth = get_auth_instance()
    initial_token = auth.get_token()
    assert initial_token is not None

    # Check if the token returned is the same as initial token since it isn't expired
    token2 = auth.get_token()
    assert token2 is not None
    assert token2 == initial_token

    # Force token to be considered expired
    auth._next_token_time = time.time() - 1  # type: ignore
    # Simulate waiting for the token to expire
    time.sleep(2)

    updated_token = auth.get_token()
    assert updated_token is not None
    assert updated_token != initial_token



from typing import Iterable, Dict
import requests
from .adaptive_types import JsonObject, RecordId, RecordIdList
from .adaptive_auth import AdaptiveAuth


class AdaptiveOapif:
    """Class to interact with Adaptive OGC API - Features endpoints."""

    _tenant: str | None
    _adaptive_auth: AdaptiveAuth | None
    _url: str

    def __init__(
        self,
        url: str,
        adaptive_auth: AdaptiveAuth | None = None,
        tenant: str | None = None,
    ):
        """Initialize the AdaptiveOgcApi class.

        Args:
            url (str): The base URL for the API.
            adaptive_auth (AdaptiveAuth, optional): AdaptiveAuth object configured with the necessary credentials. Defaults to None.
            tenant (str, optional): Adaptive tenant ID. Only required with internal URL.
        """
        self._url = url.rstrip("/") + "/api/core/v1/ogc/features/collections/"
        self._adaptive_auth = adaptive_auth
        self._tenant = tenant

    def _get_headers(self) -> Dict[str, str]:
        head = {"Content-Type": "application/json"}
        if self._adaptive_auth is not None:
            token = self._adaptive_auth.get_token()
            head["Authorization"] = f"Bearer {token}"

        if self._tenant is not None:
            head["X-Gaia-Tenant"] = self._tenant

        return head

    def get_collections(
        self,
    ):
        """Get collections from OGC API

        Returns:
            requests.Response: HTTPS response containing the available collections in the OGC API based on permissions.
        """
        headers = self._get_headers()
        return requests.get(self._url, headers=headers)

    def get_schema(
        self,
        ident: str,
    ) -> requests.Response:
        """Get schema for a specific collection

        Args:
            ident (str): Dataset alias or id

        Returns:
            requests.Response: HTTP response containing the schema for the specified collection.
        """
        headers = self._get_headers()

        return requests.get(
            self._url + ident + "/schema",
            headers=headers,
            timeout=10,
        )

    def get_items_geojson(
        self,
        ident: str,
        bbox: Iterable[float] | None = None,
        bbox_crs: str | None = None,
        datetime: str | None = None,
        filter: str | None = None,
        filter_crs: str | None = None,
        limit: int = 1000,
        offset: int = 0,
        crs: str | None = None,
        properties: Iterable[str] | None = None,
    ) -> requests.Response:
        """Get items from a collection in GeoJSON format

        Args:
            ident (str): Dataset alias or id
            bbox (Iterable[float], optional): Bounding box to filter items. Defaults to None.
            bbox_crs (str, optional): CRS of the bounding box. Defaults to None.
            datetime (str, optional): DateTime filter in ISO 8601 format. Defaults to None.
            filter (str, optional): Additional filter expression.
            filter_crs (str, optional): CRS for the filter expression. Defaults to None.
            limit (int, optional): Maximum number of items to return. Defaults to 1000.
            offset (int, optional): Offset for pagination. Defaults to 0.
            crs (str, optional): Desired CRS for the returned items. Defaults to None, which implies EPSG:4326.
            properties (Iterable[str], optional): List of properties to include in the response. Defaults to None.

        Returns:
            requests.Response: HTTP response containing the items from the specified collection in GeoJSON format.
        """
        headers = self._get_headers()

        return requests.get(
            self._url + ident + "/items",
            headers=headers,
            timeout=10,
            params={
                "bbox": bbox,
                "bbox-crs": bbox_crs,
                "datetime": datetime,
                "filter": filter,
                "filter-crs": filter_crs,
                "limit": limit,
                "offset": offset,
                "crs": crs,
                "properties": ",".join(properties) if properties else None,
            },
        )

    def get_items(
        self,
        ident: str,
        bbox: Iterable[float] | None = None,
        bbox_crs: str | None = None,
        datetime: str | None = None,
        filter: str | None = None,
        filter_crs: str | None = None,
        limit: int = 1000,
        offset: int = 0,
        crs: str | None = None,
        properties: Iterable[str] | None = None,
    ) -> requests.Response:
        """Get items from a collection

        Args:
            ident (str): Dataset alias or id
            bbox (Iterable[float], optional): Bounding box to filter items. Defaults to None.
            bbox_crs (str, optional): CRS of the bounding box. Defaults to None.
            datetime (str, optional): DateTime filter in ISO 8601 format. Defaults to None.
            filter (str, optional): Additional filter expression. Defaults to None.
            filter_crs (str, optional): CRS for the filter expression. Defaults to None.
            limit (int, optional): Maximum number of items to return. Defaults to 1000.
            offset (int, optional): Offset for pagination. Defaults to 0.
            crs (str, optional): Desired CRS for the returned items. Defaults to None, which returns the geometries in their native CRS (schema lookup is necessary).
            properties (Iterable[str], optional): List of properties to include in the response. Defaults to None.

        Returns:
            requests.Response: HTTP response containing the items from the specified collection.
        """
        headers = self._get_headers()

        return requests.get(
            self._url + ident + "/items.json",
            headers=headers,
            timeout=10,
            params={
                "bbox": bbox,
                "bbox-crs": bbox_crs,
                "datetime": datetime,
                "filter": filter,
                "filter-crs": filter_crs,
                "limit": limit,
                "offset": offset,
                "crs": crs,
                "properties": ",".join(properties) if properties else None,
            },
        )

    def get_item_geojson(
        self,
        ident: str,
        id: RecordId,
        crs: str | None = None,
        include_geometries: bool = False,
        properties: Iterable[str] | None = None,
        geometry: str | None = None,
    ) -> requests.Response:
        """Get a specific item from a collection in GeoJSON format

        Args:
            ident (str): Dataset alias or id
            id (RecordId): The ID of the item to retrieve.
            crs (str, optional): Desired CRS for the returned item. Defaults to None.
            include_geometries (bool, optional): Whether to include non-primary geometries in the response. Defaults to False. If True, additional geometries will be included as properties in the GeoJSON response, as GeoJSON geometries.
            properties (Iterable[str], optional): List of properties to include in the response. Defaults to None, which includes all properties.
            geometry (str, optional): Geometry filter in WKT format. Defaults to None.

        Returns:
            requests.Response: HTTP response containing the specified item in GeoJSON format.
        """
        headers = self._get_headers()

        return requests.get(
            self._url + ident + "/items/" + str(id),
            headers=headers,
            params={
                "crs": crs,
                "include-geometries": include_geometries,
                "properties": properties,
                "geometry": geometry,
            },
        )

    def get_item(
        self,
        ident: str,
        id: RecordId,
        crs: str | None = None,
        include_geometries: bool = True,
        properties: Iterable[str] | None = None,
        geometry: str | None = None,
    ) -> requests.Response:
        """Get a specific item from a collection

        Args:
            ident (str): Dataset alias or id
            id (RecordId): The ID of the item to retrieve.
            crs (str, optional): Desired CRS for the returned item. Defaults to None.
            include_geometries (bool, optional): Whether to include geometries in the response. Defaults to True.
            properties (Iterable[str], optional): List of properties to include in the response. Defaults to None, which includes all properties.
            geometry (str, optional): Geometry filter in WKT format. Defaults to None.

        Returns:
            requests.Response: HTTP response containing the specified item.
        """
        headers = self._get_headers()

        return requests.get(
            self._url + ident + "/items/" + str(id) + ".json",
            headers=headers,
            params={
                "crs": crs,
                "include-geometries": include_geometries,
                "properties": properties,
                "geometry": geometry,
            },
        )

    def post_item(self, ident: str, data: JsonObject) -> requests.Response:
        """Create a new item in the specified collection.

        Args:
            ident (str): Dataset alias or id
            data (dict): The item data to create. This must be a JSON object with properties matching the collection schema.

        Returns:
            requests.Response: HTTP response containing the created item id.
        """
        headers = self._get_headers()

        return requests.post(
            self._url + ident + "/items.json", headers=headers, json=data
        )

    def put_item(self, ident: str, id: RecordId, data: JsonObject):
        """Replace an existing item in the specified collection. Note: This will overwrite the entire item.

        Args:
            ident (str): Dataset alias or id
            id (RecordId): The ID of the item to replace.
            data (dict): The new item data. This must be a JSON object with properties matching the collection schema.

        Returns:
            requests.Response: HTTP response containing the updated item.
        """
        headers = self._get_headers()

        return requests.put(
            self._url + ident + "/items/" + str(id) + ".json",
            headers=headers,
            json=data,
        )

    def patch_item(self, ident: str, id: RecordId, data: JsonObject):
        """Update an existing item in the specified collection. Note: This will only update the provided properties, properties set to None will be set to null.

        Args:
            ident (str): Dataset alias or id
            id (RecordId): The ID of the item to update.
            data (dict): The item data to update. This must be a JSON object with properties matching the collection schema.

        Returns:
            requests.Response: HTTP response containing the updated item.
        """
        headers = self._get_headers()

        return requests.patch(
            self._url + ident + "/items/" + str(id) + ".json",
            headers=headers,
            json=data,
        )

    def delete_item(self, ident: str, id: RecordId):
        """Delete an item from the specified collection. Note: Please use the delete_items method for bulk deletes.

        Args:
            ident (str): Dataset alias or id
            id (RecordId): The ID of the item to delete.

        Returns:
            requests.Response: HTTP response indicating the result of the delete operation.
        """
        headers = self._get_headers()

        return requests.delete(self._url + ident + "/items/" + str(id), headers=headers)

    def delete_items(self, alias: str, ids: RecordIdList):
        """Delete multiple items from the specified collection.

        Args:
            alias (str): Dataset alias or id
            ids (list): List of item IDs to delete.

        Returns:
            requests.Response: HTTP response indicating the result of the bulk delete operation.
        """
        headers = self._get_headers()

        jsondata = {"ids": ids}
        url = self._url + alias + "/items/delete/"
        return requests.post(url, headers=headers, json=jsondata)



from typing import Iterable
import json
import requests
from enum import StrEnum
from .adaptive_types import JsonObject, RecordId
from .adaptive_auth import AdaptiveAuth


class BatchOperationMethod(StrEnum):
    """Enum for batch operations."""

    post = "post"
    put = "put"
    patch = "patch"
    delete = "delete"


JsonRef = dict[str, str]  # {"$ref": "<resultId>"}


class BatchOperation:
    """Class representing a single batch operation."""

    operation: BatchOperationMethod
    dataset: str
    data: JsonObject | None
    id: RecordId | JsonRef | None
    resultId: str | None

    def __init__(
        self,
        operation: BatchOperationMethod,
        dataset: str,
        data: JsonObject | None = None,
        id: RecordId | JsonRef | None = None,
        resultId: str | None = None,
    ):
        self.operation = operation
        self.dataset = dataset
        self.data = data
        self.id = id
        self.resultId = resultId

    def to_dict(self) -> JsonObject:
        result: JsonObject = {
            "operation": self.operation.value,
            "dataset": self.dataset,
        }
        if self.data is not None:
            result["data"] = self.data
        if self.id is not None:
            result["id"] = self.id
        if self.resultId is not None:
            result["$resultId"] = self.resultId
        return result


class AdaptiveBatch:
    """Class to handle batch operations with Adaptive."""

    _adaptive_auth: AdaptiveAuth | None
    _tenant: str | None
    _url: str

    def __init__(
        self,
        url: str,
        adaptive_auth: AdaptiveAuth | None = None,
        tenant: str | None = None,
    ):
        """Initialize the AdaptiveBatch class.

        Args:
            url (str): The base URL for the Adaptive OGC API.
            adaptive_auth (AdaptiveAuth | None, optional): An instance of AdaptiveAuth for authentication. Defaults to None.
            tenant (str | None, optional): The tenant identifier for multi-tenant setups. Defaults to None.
        """
        self._url = url.rstrip("/") + "/api/core/v1/ogc/features/collections"
        self._adaptive_auth = adaptive_auth
        self._tenant = tenant

    def batch_raw(self, data: str) -> requests.Response:
        """Sends prepared batch operations in NDJSON format.

        Args:
            data (str): The batch operations in NDJSON format.

        Returns:
            requests.Response: The response object from the batch request.
        """
        headers = {"Content-Type": "application/ndjson"}

        if self._adaptive_auth is not None:
            token = self._adaptive_auth.get_token()
            headers["Authorization"] = f"Bearer {token}"

        return requests.patch(self._url, headers=headers, data=data)

    def batch(self, operations: Iterable[BatchOperation]) -> requests.Response:
        """Sends batch operations.

        Args:
            operations (Iterable[BatchOperation]): An iterable of BatchOperation instances.

        Returns:
            requests.Response: The response object from the batch request.
        """

        data = "\n".join([json.dumps(op.to_dict()) for op in operations])
        return self.batch_raw(data)

    def operation_post(
        self, ident: str, data: JsonObject, resultId: str | None = None
    ) -> BatchOperation:
        """Create a BatchOperation for POST (create item).

        Args:
            ident (str): The dataset alias.
            data (JsonDict): The item data to create.
            resultId (str | None, optional): An optional resultId to reference the result of this operation in subsequent operations with JSON References, and to identify result in the response. Defaults to None.

        Returns:
            BatchOperation: The created BatchOperation instance.
        """
        return BatchOperation(
            operation=BatchOperationMethod.post,
            dataset=ident,
            data=data,
            resultId=resultId,
        )

    def operation_put(
        self, ident: str, id: str | JsonRef, data: JsonObject
    ) -> BatchOperation:
        """Create a BatchOperation for PUT (update item).

        Args:
            ident (str): The dataset alias.
            id (str): The ID of the item to update.
            data (JsonDict): The updated item data.

        Returns:
            BatchOperation: The created BatchOperation instance.
        """
        return BatchOperation(
            operation=BatchOperationMethod.put, dataset=ident, id=id, data=data
        )

    def operation_patch(
        self, ident: str, id: str | JsonRef, data: JsonObject
    ) -> BatchOperation:
        """Create a BatchOperation for PATCH (partially update item).

        Args:
            ident (str): The dataset alias.
            id (str): The ID of the item to partially update.
            data (JsonDict): The partial item data.

        Returns:
            BatchOperation: The created BatchOperation instance.
        """
        return BatchOperation(
            operation=BatchOperationMethod.patch, dataset=ident, id=id, data=data
        )

    def operation_delete(self, ident: str, id: str | JsonRef) -> BatchOperation:
        """Create a BatchOperation for DELETE (delete item).

        Args:
            ident (str): The dataset alias.
            id (str): The ID of the item to delete.

        Returns:
            BatchOperation: The created BatchOperation instance.
        """
        return BatchOperation(
            operation=BatchOperationMethod.delete, dataset=ident, id=id
        )



import os
from dotenv import load_dotenv
from test_adaptive_auth import get_auth_instance
from pyadaptive.adaptive_batch import AdaptiveBatch
from pyadaptive.adaptive_oapif import AdaptiveOapif
from utils import is_success

load_dotenv()

DATASET_ALIAS = os.getenv("DATASET_ALIAS", "py-adaptive-test-dataset")


def get_instances(
    with_auth: bool = True, tenant: str | None = None
) -> tuple[AdaptiveBatch, AdaptiveOapif]:
    url = os.getenv("ADAPTIVE_URL")
    assert url is not None

    if not with_auth:
        return (AdaptiveBatch(url=url), AdaptiveOapif(url=url))

    auth = get_auth_instance()
    assert auth is not None

    return (
        AdaptiveBatch(url=url, adaptive_auth=auth, tenant=tenant),
        AdaptiveOapif(url=url, adaptive_auth=auth, tenant=tenant),
    )


def test_batch_create():
    batch, oapif = get_instances(with_auth=True, tenant="master")

    operations = [
        batch.operation_post(ident=DATASET_ALIAS, data={"title": "Test Item 1"}),
        batch.operation_post(ident=DATASET_ALIAS, data={"title": "Test Item 2"}),
    ]

    response = batch.batch(operations=operations)
    assert is_success(response.status_code)

    response_data = response.json()
    assert len(response_data) == 2

    item_ids: list[int] = []
    for result in response_data:
        assert result["status"] == 201
        item_ids.append(result["data"]["id"])

    for item_id in item_ids:
        get_response = oapif.get_item(ident=DATASET_ALIAS, id=item_id)
        assert is_success(get_response.status_code)
        item_data = get_response.json()
        assert item_data["id"] == item_id
        assert "title" in item_data
        assert item_data["title"] in ["Test Item 1", "Test Item 2"]


def test_batch_update_and_delete():
    batch, oapif = get_instances(with_auth=True, tenant="master")

    # First, create an item to update and delete
    create_response = batch.batch(
        operations=[
            batch.operation_post(ident=DATASET_ALIAS, data={"title": "Item to Update"}),
            batch.operation_post(ident=DATASET_ALIAS, data={"title": "Item to Delete"}),
        ]
    )
    assert is_success(create_response.status_code)
    created_items = create_response.json()
    item_to_update_id = created_items[0]["data"]["id"]
    item_to_delete_id = created_items[1]["data"]["id"]

    # Now, update and delete the items using batch operations
    operations = [
        batch.operation_patch(
            ident=DATASET_ALIAS,
            id=item_to_update_id,
            data={"title": "Updated Item Name"},
        ),
        batch.operation_delete(ident=DATASET_ALIAS, id=item_to_delete_id),
    ]

    response = batch.batch(operations=operations)
    assert is_success(response.status_code)

    response_data = response.json()
    assert len(response_data) == 2

    # Verify the update operation
    update_result = response_data[0]
    assert is_success(update_result["status"])

    get_response = oapif.get_item(ident=DATASET_ALIAS, id=item_to_update_id)
    assert is_success(get_response.status_code)
    item_data = get_response.json()
    assert item_data["id"] == item_to_update_id
    assert item_data["title"] == "Updated Item Name"

    # Verify the delete operation
    delete_result = response_data[1]
    assert delete_result["status"] == 204

    get_response = oapif.get_item(ident=DATASET_ALIAS, id=item_to_delete_id)
    assert get_response.status_code == 404



import time
from keycloak import KeycloakOpenID


class AdaptiveAuth:
    """Class to handle authentication with Keycloak for Adaptive OGC API."""

    _keycloak_openid: KeycloakOpenID
    _access_token: str | None
    _next_token_time: float

    def __init__(
        self, server_url: str, client_id: str, client_secret_key: str, realm_name: str
    ):
        """Initialize the AdaptiveAuth class.

        Args:
            server_url (str): The Keycloak server URL.
            client_id (str): The client ID for the Keycloak client.
            client_secret_key (str): The client secret key for the Keycloak client.
            realm_name (str): The realm name in Keycloak where the client is registered.
        """
        self._keycloak_openid = KeycloakOpenID(
            server_url=server_url,
            client_id=client_id,
            client_secret_key=client_secret_key,
            realm_name=realm_name,
        )
        self._access_token = None
        self._next_token_time = 0

    def _authenticate_client(self) -> str:
        token = self._keycloak_openid.token(grant_type="client_credentials")  # type: ignore

        self._next_token_time = time.time() + token["expires_in"] - 10
        self._access_token = token["access_token"]

        return self._access_token  # type: ignore

    def get_token(self) -> str:
        """Get or update access token.

        Returns:
            str: The existing or updated access token, depending on whether it was near expiration.
        """
        if self._access_token is None or time.time() > self._next_token_time:
            return self._authenticate_client()

        return self._access_token

    def get_token_renews_after(self) -> float | None:
        """Get the token renewal time.

        Returns:
            float | None: The time (in seconds since epoch) when the token will need to be renewed, or None if no token is set.
        """
        if self._access_token is None:
            return None

        return self._next_token_time



import os
from dotenv import load_dotenv
from test_adaptive_auth import get_auth_instance
from pyadaptive.adaptive_oapif import AdaptiveOapif
from utils import is_success

load_dotenv()

DATASET_ALIAS = os.getenv("DATASET_ALIAS", "py-adaptive-test-dataset")


def get_instance(with_auth: bool = True, tenant: str | None = None) -> AdaptiveOapif:
    url = os.getenv("ADAPTIVE_URL")
    assert url is not None

    if not with_auth:
        return AdaptiveOapif(url=url)

    auth = get_auth_instance()
    assert auth is not None

    return AdaptiveOapif(url=url, adaptive_auth=auth, tenant=tenant)


def test_header_with_auth():
    api = get_instance(with_auth=True)
    assert api is not None

    headers = api._get_headers()  # type: ignore
    assert "Authorization" in headers
    assert headers["Authorization"].startswith("Bearer ")
    assert len(headers["Authorization"]) > 10


def test_header_with_tenant():
    api = get_instance(with_auth=True, tenant="master")
    assert api is not None

    headers = api._get_headers()  # type: ignore
    assert "X-Gaia-Tenant" in headers
    assert headers["X-Gaia-Tenant"] == "master"


def test_get_collections():
    api = get_instance(with_auth=True)
    assert api is not None

    response = api.get_collections()
    assert is_success(response.status_code)

    response = response.json()

    assert "collections" in response
    assert isinstance(response["collections"], list)


def test_get_schema():
    api = get_instance(with_auth=True)
    assert api is not None

    response = api.get_schema(ident=DATASET_ALIAS)
    assert is_success(response.status_code)

    response = response.json()
    assert "properties" in response
    assert isinstance(response["properties"], dict)


def test_get_items():
    api = get_instance(with_auth=True)
    assert api is not None

    response = api.get_items(ident=DATASET_ALIAS)
    assert is_success(response.status_code)

    response = response.json()
    assert "data" in response
    assert isinstance(response["data"], list)


def test_post_item():
    api = get_instance(with_auth=True)
    assert api is not None

    item_data = {
        "title": "Testing",
        "description": "This is a description",
        "point": "POINT(58.45961 8.77323)",
    }
    response = api.post_item(ident=DATASET_ALIAS, data=item_data)

    assert is_success(response.status_code)

    json = response.json()
    assert "id" in json

    item_id = json.get("id")
    get_response = api.get_item(ident=DATASET_ALIAS, id=item_id)
    assert is_success(get_response.status_code)

    get_data = get_response.json()
    assert get_data["title"] == "Testing"
    assert get_data["description"] == "This is a description"
    assert get_data["point"] == "POINT(58.45961 8.77323)"


def test_put_item():
    api = get_instance(with_auth=True)
    assert api is not None

    # First, create an item to update
    item_data = {
        "title": "Testing for PUT",
        "description": "This is a description for PUT",
        "point": "POINT(58.45961 8.77323)",
    }
    post_response = api.post_item(ident=DATASET_ALIAS, data=item_data)
    assert is_success(post_response.status_code)
    item_id = post_response.json().get("id")
    assert item_id is not None

    # Now, update the created item
    updated_data = {
        "title": "Updated Title",
        "description": "Updated description",
        "point": "POINT(59 9)",
    }
    put_response = api.put_item(ident=DATASET_ALIAS, id=item_id, data=updated_data)
    assert is_success(put_response.status_code)

    get_response = api.get_item(ident=DATASET_ALIAS, id=item_id)
    assert is_success(get_response.status_code)

    item = get_response.json()

    assert item["title"] == "Updated Title"
    assert item["description"] == "Updated description"
    assert item["point"] == "POINT(59 9)"


def test_delete_item():
    api = get_instance(with_auth=True)
    assert api is not None

    # First, create an item to delete
    item_data = {
        "title": "Testing for DELETE",
        "description": "This is a description for DELETE",
        "point": "POINT(58.45961 8.77323)",
    }
    post_response = api.post_item(ident=DATASET_ALIAS, data=item_data)
    assert is_success(post_response.status_code)
    item_id = post_response.json().get("id")
    assert item_id is not None

    # Now, delete the created item
    delete_response = api.delete_item(ident=DATASET_ALIAS, id=item_id)
    assert is_success(delete_response.status_code)

    # Verify the item has been deleted
    get_response = api.get_item(ident=DATASET_ALIAS, id=item_id)
    assert get_response.status_code == 404



import os
from dotenv import load_dotenv
from test_adaptive_auth import get_auth_instance
from pyadaptive.adaptive_import import AdaptiveImport
from pyadaptive.adaptive_oapif import AdaptiveOapif
from utils import is_success

load_dotenv()

DATASET_ALIAS = os.getenv("DATASET_ALIAS", "py-adaptive-test-dataset")


def get_instances(
    with_auth: bool = True, tenant: str | None = None
) -> tuple[AdaptiveImport, AdaptiveOapif]:
    url = os.getenv("ADAPTIVE_URL")
    assert url is not None

    if not with_auth:
        return (AdaptiveImport(url=url), AdaptiveOapif(url=url))

    auth = get_auth_instance()
    assert auth is not None

    return (
        AdaptiveImport(url=url, adaptive_auth=auth, tenant=tenant),
        AdaptiveOapif(url=url, adaptive_auth=auth, tenant=tenant),
    )


def test_import_items():
    importer, oapif = get_instances(with_auth=True)

    items_to_import = [
        {"title": "Import Test Item 1"},
        {"title": "Import Test Item 2"},
    ]

    response = importer.import_items(
        dataset=DATASET_ALIAS,
        items=items_to_import,
        options={"truncate": True, "resetSerialPrimaryKey": True},
    )
    assert is_success(response.status_code)

    get_items_response = oapif.get_items(ident=DATASET_ALIAS)
    assert is_success(get_items_response.status_code)

    items = get_items_response.json()
    assert "data" in items
    assert len(items["data"]) == 2

    titles = [item["title"] for item in items["data"]]
    assert "Import Test Item 1" in titles
    assert "Import Test Item 2" in titles
